﻿#region Usings
    using System;
    using System.Collections.Generic;
    using System.Linq;

    using Styx;
    using Styx.WoWInternals;
    using Styx.WoWInternals.WoWObjects;

using System.Windows.Media;

using Styx.Common;
using Styx.CommonBot.Routines;
using CommonBehaviors.Actions;
using Styx.TreeSharp;
using Styx.CommonBot;
#endregion

#region Bindings
    using SB = Arcanum.Core.Helpers.Spell_Book;
    using S = Arcanum.Core.Spell;
    using D = Arcanum.Core.Helpers.Debug;
#endregion

namespace Arcanum.Core
{
    static class Unit
    {
        private static LocalPlayer Me { get { return StyxWoW.Me; } }
        private static WoWUnit currentTarget { get { return StyxWoW.Me.CurrentTarget; } }
        private static List<WoWUnit> enemyCount { get; set; }
        private static float calculateMeleeRange(WoWUnit Unit) { return !Me.GotTarget ? 0f : Math.Max(5f, Me.CombatReach + 1.3333334f + Unit.CombatReach); }
        public static bool isInMeleeRange(WoWUnit Unit) { return Unit.Distance < calculateMeleeRange(Unit); }
        public static bool isUnitValid(WoWUnit Unit) { return Unit != null && Unit.IsValid && Unit.IsAlive && Unit.Attackable; }
        private static List<WoWUnit> Enemies { get; set; }

        #region Active Enemies
        public static IEnumerable<WoWUnit> activeEnemies(WoWPoint fromLocation, double Range)
        {
            try
            {
                var Hostile = enemyCount;
                return Hostile != null ? Hostile.Where(x => x.Location.DistanceSqr(fromLocation) <= Range * Range) : null;
            }
            catch (Exception error)
            {
                D.diagnosticLog("Exception trown in activeEnemies: " + error);
                return null;
            }
        }

        public static void enemyAnnex(double Range)
        {
            try
            {
                enemyCount.Clear();
                foreach (var u in surroundingEnemies())
                {
                    if (u == null || !u.IsValid)
                        continue;
                    if (!u.IsAlive || u.DistanceSqr > Range * Range)
                        continue;
                    if (!u.Attackable || !u.CanSelect)
                        continue;
                    if (u.IsFriendly)
                        continue;
                    if (u.IsNonCombatPet && u.IsCritter)
                        continue;
                    enemyCount.Add(u);
                }
            }
            catch (Exception error)
            {
                D.diagnosticLog("Exception trown in enemyAnnex: " + error);
            }
         }

        private static IEnumerable<WoWUnit> surroundingEnemies() { return ObjectManager.GetObjectsOfTypeFast<WoWUnit>(); }

        static Unit()
        {
            enemyCount = new List<WoWUnit>();
            Enemies = new List<WoWUnit>();
            activeAoEEnemyCount = new List<WoWUnit>();
            activeEnemyCount = new List<WoWUnit>();
        }

        #endregion

        #region Aura Detection
        public static bool auraExists(WoWUnit Unit, int auraID, bool isMyAura = false)
        {
            try
            {
                if (Unit == null || !Unit.IsValid)
                    return false;
                WoWAura Aura = isMyAura ? Unit.GetAllAuras().FirstOrDefault(A => A.SpellId == auraID && A.CreatorGuid == Me.Guid) : Unit.GetAllAuras().FirstOrDefault(A => A.SpellId == auraID);
                return Aura != null;
            }
            catch (Exception error)
            {
                D.diagnosticLog("Exception trown in auraExists: " + error);
                return false;
            }
        }

        public static uint auraStacks(WoWUnit Unit, int auraID, bool isMyAura = false)
        {
            try
            {
                if (Unit == null || !Unit.IsValid)
                    return 0;
                WoWAura Aura = isMyAura ? Unit.GetAllAuras().FirstOrDefault(A => A.SpellId == auraID && A.CreatorGuid == Me.Guid) : Unit.GetAllAuras().FirstOrDefault(A => A.SpellId == auraID);
                return Aura != null ? Aura.StackCount : 0;
            }
            catch (Exception error)
            {
                D.diagnosticLog("Exception trown in auraStacks: " + error);
                return 0;
            }
        }

        public static double auraTimeLeft(WoWUnit Unit, int auraID, bool isMyAura = false)
        {
            try
            {
                if (Unit == null || !Unit.IsValid)
                    return 0;
                WoWAura Aura = isMyAura ? Unit.GetAllAuras().FirstOrDefault(A => A.SpellId == auraID && A.CreatorGuid == Me.Guid) : Unit.GetAllAuras().FirstOrDefault(A => A.SpellId == auraID);
                return Aura != null ? Aura.TimeLeft.TotalMilliseconds : 0;
            }
            catch (Exception error)
            {
                D.diagnosticLog("Exception trown in auraTimeLeft: " + error);
                return 9999;
            }
        }
        #endregion

        #region Unit Evaluation
        public static double luaGetPower()
        {
            try
            {
                using (StyxWoW.Memory.AcquireFrame()) { return Lua.GetReturnVal<int>("return UnitPower(\"player\");", 0); }
            }
            catch (Exception error)
            {
                D.diagnosticLog("Exception trown in luaGetPower: " + error);
                return Me.CurrentPower;
            }
        }

        public static float luaGetRegen()
        {
            try
            {
                using (StyxWoW.Memory.AcquireFrame()) { return Lua.GetReturnVal<float>("return GetPowerRegen();", 1); }
            }
            catch (Exception error)
            {
                D.diagnosticLog("Exception trown in luaGetRegen: " + error);
                return 4;
            }
        }

        #endregion

        #region Enemies
        public static List<WoWUnit> GetEnemies()
        {
            return Enemies;
        }

        private static IEnumerable<WoWUnit> SurroundingUnits()
        {
            return ObjectManager.GetObjectsOfTypeFast<WoWUnit>();
        }

        internal static bool IsDummy(this WoWUnit unit)
        {
            return unit.Name.Contains("Training Dummy");
        }
        internal static bool IsCrystal(this WoWUnit unit)
        {
            return unit.Name.Contains("Prismatic Crystal");
        }

        public static void UpdateEnemyList(double range)
        {
            try
            {
                Enemies.Clear();
                foreach (var unit in SurroundingUnits().Where(unit => isUnitValid(unit) && !unit.IsFriendly && unit.Attackable
                    && (unit.DistanceSqr < range * range) && unit.InLineOfSpellSight && (unit.Combat || IsCrystal(unit) || IsDummy(unit))))
                {
                    Enemies.Add(unit);
                }
            }
            catch (Exception error)
            {
                D.diagnosticLog("Exception trown in UpdateEnemyList: " + error);
            }
        }

        #endregion

        #region Cache Auras, Stacks & Tier peices

        public static int tierPieces { get { return countTier(new List<uint> { 115550, 115551, 115552, 115553, 115554 }); } }
        private static readonly List<uint> numberEquipped = new List<uint>();
        public static uint stacksArcaneCharge;
        public static uint stacksArcaneMissile;
        public static uint stacksFingerFrost;
        public static uint stacksBrainFreeze;
        
        public static void Cache()
        {
            stacksArcaneCharge = auraStacks(Me, SB.auraArcaneCharge);
            stacksArcaneMissile = auraStacks(Me, SB.auraArcaneMissile);
            stacksFingerFrost = auraStacks(Me, SB.auraFingerFrost);
            stacksBrainFreeze = auraStacks(Me, SB.auraBrainFreeze);
        }

        internal static int countTier(List<uint> itemID)
        {
            return numberEquipped.Count(itemID.Contains);
        }

        public static void checkGear (object sender = null, LuaEventArgs args = null)
        {
            numberEquipped.Clear();

            for (uint i = 0; i < 18; i++)
            {
                var itemInSlot = StyxWoW.Me.Inventory.GetItemBySlot(i);

                if (itemInSlot != null)
                    numberEquipped.Add(itemInSlot.ItemInfo.Id);
            }
        }

        #endregion

        #region AOE checks
        private static IEnumerable<WoWUnit> surroundingActiveEnemies() { return ObjectManager.GetObjectsOfTypeFast<WoWUnit>(); }
        private static IEnumerable<WoWUnit> surroundingAoEEnemies() { return ObjectManager.GetObjectsOfTypeFast<WoWUnit>(); }

        private static List<WoWUnit> activeAoEEnemyCount { get; set; }
        private static List<WoWUnit> activeEnemyCount { get; set; }

        public static int activeCountAoE { get { return activeAoEEnemyCount.Count; } }

        public static int activeCount { get { return activeEnemyCount.Count; } }

        public static void activeEnemyAnnex(double Range)
        {
            try
            {
                activeEnemyCount.Clear();
                foreach (var u in surroundingActiveEnemies())
                {
                    if (!isUnitValid(u))
                        continue;
                    if (!u.Attackable || !u.CanSelect)
                        continue;
                    if (!u.Combat)
                        continue;
                    if (!u.IsAlive || u.DistanceSqr > Range * Range)
                        continue;
                    if (!Me.Aggro)
                        activeEnemyCount.Add(u);
                }
            }
            catch (Exception error)
            {
                D.diagnosticLog("Exception trown in activeEnemyAnnex: " + error);
            }
        }

        public static void AoEenemyAnnex(double Range)
        {
            try
            {
                activeAoEEnemyCount.Clear();
                foreach (var u in surroundingAoEEnemies())
                {
                    if (!isUnitValid(u))
                        continue;
                    if (!u.Attackable || !u.CanSelect)
                        continue;
                    if (!u.Combat)
                        continue;
                    if (!u.IsAlive || u.DistanceSqr > Range * Range)
                        continue;
                    if (!Me.Aggro)
                        activeAoEEnemyCount.Add(u);
                }
            }
            catch (Exception error)
            {
                D.diagnosticLog("Exception trown in AoEenemyAnnex: " + error);
            }
        }

        #endregion

        #region Group and Raid checks

        public static IEnumerable<WoWUnit> PartyMembers
        {
            get
            {
                return SearchAreaPlayers();
            }
        }

        public static IEnumerable<WoWPlayer> SearchAreaPlayers()
        {
            try
            {
                return ObjectManager.GetObjectsOfTypeFast<WoWPlayer>().Where(p => p != null && p.IsInMyPartyOrRaid);
            }
            catch (Exception error)
            {
                D.diagnosticLog("Exception trown in SearchAreaPlayers: " + error);
                return null;
            }
        }

        #endregion
    }
}
